from enum import Enum, IntEnum

DEBUG_FREQ_MULTIPLIER: int = 100


class VehicleSegment(Enum):
    Chassis = 0
    PowerTrain = 1
    Body = 2


class Frequency(float, Enum):
    VeryHigh = 0.01 * DEBUG_FREQ_MULTIPLIER
    High = 0.05 * DEBUG_FREQ_MULTIPLIER
    Low = 0.1 * DEBUG_FREQ_MULTIPLIER
    VeryLow = 0.5 * DEBUG_FREQ_MULTIPLIER


class CANFrame(IntEnum):
    BrakeOperationIndicator = 0x1A
    BrakeOutputIndicator = 0x24
    BrakeOilIndicator = 0x146

    AntiLockBrakeOperation = 0x15A

    AcceleratorPedalOperationIndicator = 0x2F
    ThrottlePosition = 0x39
    ThrottleAdjustment = 0x16F
    EngineRPMOrSpeed = 0x43

    EngineCoolingWaterTemperature = 0x183

    EngineMalfunction = 0x18D

    SteeringWheelOperationPosition = 0x58
    PowerSteeringOutputIndicator = 0x62
    PowerSteeringMalfunction = 0x198

    ShiftPositionSwitch = 0x6D
    ShiftPosition = 0x77

    EngineStartButton = 0x1B8
    EngineStatus = 0x19A
    FuelAmount = 0x3D4
    BatteryWarning = 0x3DE

    RightLeftTurnSignalOrHazardSwitch = 0x83
    TurnSignalIndicator = 0x8D

    HornSwitch = 0x98
    HornOperation = 0xA2

    PositionOrHeadLightsOrHighBeamSwitch = 0x1A7
    HeadlightFlashingSwitch = 0x1B1
    PositionOrHeadLightsOrHighBeamIndicator = 0x1BB

    FrontWiperOrWasherSwitch = 0x25C
    FrontWiperOrWasherStatus = 0x266

    RearWiperOrWasherSwitch = 0x271
    RearWiperOrWasherStatus = 0x27B

    DoorsLockOrUnlockSwitch = 0x286
    DoorsOpenClosedOrLockedUnlockedIndicator = 0x290
    DoorLockDriveUnitMalfunction = 0x420

    ParkingBrake = 0x1C9
    ParkingBrakeStatus = 0x1D3

    RightDoorOrWindowLiftingSwitch = 0x29C
    RightDoorOrWindowPositionOrLimitSwitchStatus = 0x2A6

    LeftDoorOrWindowLiftingSwitch = 0x2B1
    LeftDoorOrWindowPositionOrLimitSwitchStatus = 0x2BB

    AirbagActivationSwitch = 0xB4

    SeatBeltSensor = 0x457
    SeatBeltAlarm = 0x461

    BonnetHoodOpenOrCloseSwitch = 0x46C

    TrunkOpenOrCloseSwitch = 0x477

    EcoDrivingJudgement = 0x482

    @property
    def source(self) -> VehicleSegment:
        match self:
            case CANFrame.BrakeOperationIndicator \
                 | CANFrame.AcceleratorPedalOperationIndicator \
                 | CANFrame.SteeringWheelOperationPosition \
                 | CANFrame.ShiftPositionSwitch \
                 | CANFrame.EngineStartButton \
                 | CANFrame.RightLeftTurnSignalOrHazardSwitch \
                 | CANFrame.HornSwitch \
                 | CANFrame.PositionOrHeadLightsOrHighBeamSwitch \
                 | CANFrame.HeadlightFlashingSwitch \
                 | CANFrame.FrontWiperOrWasherSwitch \
                 | CANFrame.RearWiperOrWasherSwitch \
                 | CANFrame.DoorsLockOrUnlockSwitch \
                 | CANFrame.ParkingBrake \
                 | CANFrame.RightDoorOrWindowLiftingSwitch \
                 | CANFrame.LeftDoorOrWindowLiftingSwitch:
                return VehicleSegment.Chassis
            case CANFrame.TurnSignalIndicator \
                 | CANFrame.HornOperation \
                 | CANFrame.PositionOrHeadLightsOrHighBeamIndicator \
                 | CANFrame.FrontWiperOrWasherStatus \
                 | CANFrame.RearWiperOrWasherStatus \
                 | CANFrame.DoorsOpenClosedOrLockedUnlockedIndicator \
                 | CANFrame.DoorLockDriveUnitMalfunction \
                 | CANFrame.RightDoorOrWindowPositionOrLimitSwitchStatus \
                 | CANFrame.LeftDoorOrWindowPositionOrLimitSwitchStatus \
                 | CANFrame.AirbagActivationSwitch \
                 | CANFrame.SeatBeltAlarm \
                 | CANFrame.SeatBeltSensor \
                 | CANFrame.BonnetHoodOpenOrCloseSwitch \
                 | CANFrame.TrunkOpenOrCloseSwitch:
                return VehicleSegment.Body
            case _:
                return VehicleSegment.PowerTrain

    @property
    def execution(self) -> VehicleSegment:
        match self:
            case CANFrame.BrakeOperationIndicator \
                 | CANFrame.AcceleratorPedalOperationIndicator \
                 | CANFrame.SteeringWheelOperationPosition \
                 | CANFrame.ShiftPositionSwitch \
                 | CANFrame.EngineStartButton \
                 | CANFrame.ParkingBrake:
                return VehicleSegment.PowerTrain
            case CANFrame.RightLeftTurnSignalOrHazardSwitch \
                 | CANFrame.HornSwitch \
                 | CANFrame.PositionOrHeadLightsOrHighBeamSwitch \
                 | CANFrame.HeadlightFlashingSwitch \
                 | CANFrame.FrontWiperOrWasherSwitch \
                 | CANFrame.RearWiperOrWasherSwitch \
                 | CANFrame.DoorsLockOrUnlockSwitch \
                 | CANFrame.RightDoorOrWindowLiftingSwitch \
                 | CANFrame.LeftDoorOrWindowLiftingSwitch:
                return VehicleSegment.Body
            case _:
                return VehicleSegment.Chassis

    @property
    def interval(self) -> float:
        match self:
            case CANFrame.BrakeOperationIndicator \
                 | CANFrame.BrakeOutputIndicator \
                 | CANFrame.AcceleratorPedalOperationIndicator \
                 | CANFrame.ThrottlePosition \
                 | CANFrame.EngineRPMOrSpeed \
                 | CANFrame.SteeringWheelOperationPosition \
                 | CANFrame.PowerSteeringOutputIndicator \
                 | CANFrame.ShiftPositionSwitch \
                 | CANFrame.ShiftPosition \
                 | CANFrame.RightLeftTurnSignalOrHazardSwitch \
                 | CANFrame.TurnSignalIndicator \
                 | CANFrame.HornSwitch \
                 | CANFrame.HornOperation \
                 | CANFrame.AirbagActivationSwitch:
                return Frequency.VeryHigh
            case CANFrame.BrakeOilIndicator \
                 | CANFrame.AntiLockBrakeOperation \
                 | CANFrame.ThrottleAdjustment \
                 | CANFrame.EngineCoolingWaterTemperature \
                 | CANFrame.EngineMalfunction \
                 | CANFrame.PowerSteeringMalfunction \
                 | CANFrame.EngineStartButton \
                 | CANFrame.EngineStatus \
                 | CANFrame.PositionOrHeadLightsOrHighBeamSwitch \
                 | CANFrame.HeadlightFlashingSwitch \
                 | CANFrame.PositionOrHeadLightsOrHighBeamIndicator \
                 | CANFrame.ParkingBrake \
                 | CANFrame.ParkingBrakeStatus:
                return Frequency.High
            case CANFrame.FrontWiperOrWasherSwitch \
                 | CANFrame.FrontWiperOrWasherStatus \
                 | CANFrame.RearWiperOrWasherSwitch \
                 | CANFrame.RearWiperOrWasherStatus \
                 | CANFrame.DoorsLockOrUnlockSwitch \
                 | CANFrame.DoorsOpenClosedOrLockedUnlockedIndicator \
                 | CANFrame.RightDoorOrWindowLiftingSwitch \
                 | CANFrame.RightDoorOrWindowPositionOrLimitSwitchStatus \
                 | CANFrame.LeftDoorOrWindowLiftingSwitch \
                 | CANFrame.LeftDoorOrWindowPositionOrLimitSwitchStatus:
                return Frequency.Low
            case CANFrame.FuelAmount \
                 | CANFrame.BatteryWarning \
                 | CANFrame.DoorLockDriveUnitMalfunction \
                 | CANFrame.SeatBeltSensor \
                 | CANFrame.SeatBeltAlarm \
                 | CANFrame.BonnetHoodOpenOrCloseSwitch \
                 | CANFrame.TrunkOpenOrCloseSwitch \
                 | CANFrame.EcoDrivingJudgement:
                return Frequency.VeryLow
